#!/usr/bin/perl -w
## ######### PROJECT NAME : ##########
## 
## genere_cle.pl for Generation de cle GPG in /var/www/proj/my_aptget/husiana_gpg/Newscript
## 
## ######### PROJECT DESCRIPTION : ###
##
##	Ce script creer les cles publique et privee du client,
## puis il les ajoutes a son trousseau.
## Ce script a pr but d'etre execute une seule fois, et
## supprime ensuite.
## 
## 
## ###################################
## 
## Made by husianycia cedric
## Login   <husiana@intellique.com>
## 
## Started on  Fri Jan 21 13:36:58 CET 2005 husianycia cedric
## Last update Thu May 26 16:57:37 2005 husianycia cedric
## 
## ###################################
##
## Version :

my	$Version = "v1.01-2";

#############################################################
## Libs

use	strict;
use	Digest::MD5 qw(md5_hex);	#cryptage de chaine
use	GnuPG qw( :algo );		#creation et import/export de cles
use	Config::IniFiles;		#utilisation de fichier de conf.ini

#############################################################
## Variables

my	$inifile		= "/etc/storiq/genere_cle.ini";
		# fichier de configuration
my	$serial_number_file	= "";
		#Nom du fichier contenant le numero de serie
my	$export_key_directory	= "";
		#Chemin ou mettre la cle publique du client
my	$homedir		= "";
		#chemin de gnupg
my	$cfg_algo		= "";
		
my	$cfg_email		= "";

my	$cfg_size		= "";

my	$serial_number		= "";
		#numero de serie du client
my	$gpg			= "";

my	$export_key_file	= "";
		#chemin et nom complet de la cle a exporter
my	$log_file		= "";
		#nom du fichier de log


#############################################################
## Main

sub	main()
{
    # print "recupere les variables ds le fichier de conf\n";
    config_variables($inifile);
    # print "parse le numero de serie du client $serial_number_file \n";
    $serial_number = parse_serial_number($serial_number_file);
    # print "Num de serie : ".$serial_number."\n";
    $gpg = new GnuPG(homedir => $homedir);
    # print "genere les cles client : $log_file\n";
    generate_key($cfg_algo, $cfg_size, 
		 $serial_number, $cfg_email, 
		 $serial_number, capture_Mac_Address());
    # print "creer le nom ou est exportee la cle\n";
    $export_key_file = $export_key_directory . $serial_number . ".gpg";
    # print "exporte la cle\n";
    export_key($serial_number, $export_key_file);
}

#############################################################
## parsing du fichier contenant le serial number

sub parse_serial_number {
    my $config = shift;
    my @settings_serial_number = ();
    my $serial_number;
    
    
    print_error("ERROR $config does not exist.\n") if (!(-e $config));
    print_error("ERROR reading $config\n") if (!open(filehandle1, $config));
    
    @settings_serial_number = <filehandle1>;
    
    close (filehandle1);
    
    for (my $i = 0; $i < ($#settings_serial_number + 1); $i++){
	$settings_serial_number[$i] = "" if ($settings_serial_number[$i] =~ /^\#/);
	if ($settings_serial_number[$i] !~ /^\s*$/) {
	    $settings_serial_number[$i] =~ s/\s*//g;
	    $serial_number = $settings_serial_number[$i];
	}
    }
    
    print_error("ERROR pas de numero de serie\n") if (!$serial_number);
    
    return $serial_number;
}

############################################################
## Fonction qui exporte la cle vers le chemin donne

sub export_key {
    my ($name, $output) = @_;
    
    open STDOUT, ">/dev/null" 
	or print_error("ERROR Redirection impossible.\n");
    select STDOUT;
    
    eval { 
	$gpg->export_keys(keys => $name, armor => 1, output => $output);
    };
    
    close STDOUT;
    
    if ( $@ ) {
	my ($variable, @tableau) = split (" at", $@);
	
	print_error("ERROR $variable");
    }
}

#############################################################
## Fonction de retour d'erreur ds le fichier de log

sub	print_error
{
    my @date = `date`;
    @date = split("\n", $date[0]);
    open (LOG, ">>$log_file");
    print LOG "Update Intellique on $date[0] : $_[0]\n";
    close LOG;
    print "Une erreur est survenue!\n";
    exit 1;
}

#############################################################
## Generation des cles GPG du client

sub generate_key {
    my ($algo, $size, $name, $email, $comment, $secret_phrase) = @_;
    
    
    open STDOUT, ">/dev/null" or print_error("ERROR Redirection impossible.\n");
    select STDOUT;
    
    eval { 
	$gpg->gen_key(algo => $algo, size => $size, valid => 0, name => $name, email => $email, comment => $comment, passphrase => $secret_phrase);
    };
    
    close STDOUT;
    
    if ( $@ ) {
	my ($variable, @tableau) = split (" at", $@);
	
	print_error("ERROR GPG $variable");
    }
}

#############################################################
## Concatenation des Mac_adress afin de crer la passphrase

sub capture_Mac_Address {
    my $mac_eth0 = qx#/sbin/ifconfig eth0 2> /dev/null#;
    my $mac_eth1 = qx#/sbin/ifconfig eth1 2> /dev/null#;
    
    if ($mac_eth0 =~ /HWaddr (\w+:\w+:\w+:\w+:\w+:\w+)/) { $mac_eth0 = $1;}
    else { $mac_eth0 = "00:00:00:00:00:00";}
    
    if ($mac_eth1 =~ /HWaddr (\w+:\w+:\w+:\w+:\w+:\w+)/) { $mac_eth1 = $1;}
    else { $mac_eth1 = "00:00:00:00:00:00";}
    
    return (md5_hex($mac_eth0) . md5_hex($mac_eth1));
}

#############################################################
## Recuperation des variables ds fichier .ini

sub config_variables {
    my $config_file = shift;
    
    my $cfg		= new Config::IniFiles( -file => "$config_file" ) 
	or print_error("ERROR can't open ini file $config_file \n");
    my $racine	= $cfg->val("general", "racine");
    
    
    my $update_intellique = $cfg->val("general", "update_intellique");
    my $cfg_update_intellique = new Config::IniFiles( -file => "$update_intellique" ) or print_error("ERROR can't open ini file $update_intellique \n");
    
    $serial_number_file = $racine . 
	$cfg_update_intellique->val("general", "serial_number_file");
    $export_key_directory = $racine . $cfg_update_intellique->val("general", "export_key_directory");
    $homedir	= $racine . $cfg_update_intellique->val("general", 
							"homedir");
    
    $log_file	= $racine . $cfg_update_intellique->val("general",
							"log_file");
    $cfg_algo	= $cfg->val("genere_key", "algo");
    $cfg_email	= $cfg->val("genere_key", "email");
    
    $cfg_size	= $cfg->val("genere_key", "size");
}

#############################################################
## Appel du main

main();
